%define major 3
%define minor 8
%define fix 0

Name: lapack
Version: %{major}.%{minor}.%{fix}
Release: 2
License: BSD-3-Clause
Summary: LAPACK - Linear Algebra PACKage
Url: http://www.netlib.org/lapack

Source0: http://www.netlib.org/lapack/lapack-%{version}.tar.gz

Source1: static-to-shared.sh

Patch0: lapack-missing-symbol.patch

BuildRequires: gcc-gfortran
BuildRequires: sed-gnu

%description
liblapack is a math libarary.

%package -n liblapack%{major}
Summary: %{summary}
Requires(post): update-alternatives
Requires(postun): update-alternatives
Requires: libblas%{major} = %{version}
# Needed because automatic dependency generation doesn’t work with symlinks
Provides: lib:%{_libdir}/liblapack-indirect.so.0(shr_64.o)(ppc64)

%description -n liblapack%{major}
liblapack is a math libarary.

%package -n liblapacke%{major}
Summary: The C language interface to LAPACK
Requires(post): update-alternatives
Requires(postun): update-alternatives
Requires: liblapack%{major} = %{version}
# Needed because automatic dependency generation doesn’t work with symlinks
Provides: lib:%{_libdir}/liblapacke-indirect.so.0(shr_64.o)(ppc64)

%description -n liblapacke%{major}
The C language interface to LAPACK

%package -n libblas%{major}
Summary: Basic Linear Algebra Subprograms
Requires(post): update-alternatives
Requires(postun): update-alternatives
# Needed because automatic dependency generation doesn’t work with symlinks
Provides: lib:%{_libdir}/libblas-indirect.so.0(shr_64.o)(ppc64)

%description -n libblas%{major}
The BLAS (Basic Linear Algebra Subprograms) are routines that provide standard
building blocks for performing basic vector and matrix operations.

%package -n libcblas%{major}
Summary: The C language interface of BLAS
Requires(post): update-alternatives
Requires(postun): update-alternatives
Requires: libblas%{major} = %{version}
# Needed because automatic dependency generation doesn’t work with symlinks
Provides: lib:%{_libdir}/libcblas-indirect.so.0(shr_64.o)(ppc64)

%description -n libcblas%{major}
The C language interface of BLAS


%package devel
Summary: LAPACK development files
Requires: liblapack%{major} = %{version}

%description devel
LAPACK development files

%package -n lapacke-devel
Summary: LAPACKE development files
Requires: liblapacke%{major} = %{version}

%description -n lapacke-devel
LAPACKE development files

%package -n blas-devel
Summary: BLAS development files
Requires: libblas%{major} = %{version}

%description -n blas-devel
BLAS development files

%package -n cblas-devel
Summary: CBLAS development files
Requires: libcblas%{major} = %{version}

%description -n cblas-devel
CBLAS development files


%prep
%setup -q -n lapack-%{version}

%patch0 -p1

%build
cp make.inc.example make.inc

CFLAGS="-O3 -fPIC"
OPTS="-O2 -frecursive -fPIC"
NOOPT=$OPTS
LOADOPTS="-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib"

# Build BLAS
%make_build blaslib CFLAGS="$CFLAGS" OPTS="$OPTS" NOOPT="$NOOPT" LOADOPTS="$LOADOPTS"
CC=gfortran %{SOURCE1} libblas.so.%{major} librefblas.a $LOADOPTS

ar -X64 x libblas.so.%{major}

sed -i 's/libblas.so.%{major}/libblas-indirect.so.0/' shr_64.imp
        
cp shr_64.imp libblas.imp

# Build CBLAS
%make_build cblaslib CFLAGS="$CFLAGS" OPTS="$OPTS" NOOPT="$NOOPT" LOADOPTS="$LOADOPTS"
CC=gfortran %{SOURCE1} libcblas.so.%{major} libcblas.a ./libblas.so.%{major} $LOADOPTS

ar -X64 x libcblas.so.%{major}

sed -i 's/libcblas.so.%{major}/libcblas-indirect.so.0/' shr_64.imp

# Remove optional BLAS symbols implement in Netlib, but not OpenBLAS
# to provide a common interface supported by both

sed -i -e '/cdotcsub_/d' \
        -e '/cdotusub_/d'  \
        -e '/dasumsub_/d' \
        -e '/ddotsub_/d' \
        -e '/dnrm2sub_/d' \
        -e '/dsdotsub_/d' \
        -e '/dzasumsub_/d' \
        -e '/dznrm2sub_/d' \
        -e '/icamaxsub_/d' \
        -e '/idamaxsub_/d' \
        -e '/isamaxsub_/d' \
        -e '/izamaxsub_/d' \
        -e '/sasumsub_/d' \
        -e '/scasumsub_/d' \
        -e '/scnrm2sub_/d' \
        -e '/sdotsub_/d' \
        -e '/sdsdotsub_/d' \
        -e '/snrm2sub_/d' \
        -e '/zdotcsub_/d' \
        -e '/zdotusub_/d' \
        shr_64.imp

# Remove Netlib-specific symbols from the common interface

sed -i  -e '/CBLAS_CallFromC/d' \
        -e '/RowMajorStrg/d' \
        shr_64.imp


cp shr_64.imp libcblas.imp

# Build LAPACK
%make_build lapacklib CFLAGS="$CFLAGS" OPTS="$OPTS" NOOPT="$NOOPT" LOADOPTS="$LOADOPTS"
CC=gfortran %{SOURCE1} liblapack.so.%{major} liblapack.a ./libblas.so.%{major} $LOADOPTS

ar -X64 x liblapack.so.%{major}

sed -i 's/liblapack.so.%{major}/liblapack-indirect.so.0/' shr_64.imp
        
cp shr_64.imp liblapack.imp

# Build LAPACKE
%make_build lapackelib CFLAGS="$CFLAGS" OPTS="$OPTS" NOOPT="$NOOPT" LOADOPTS="$LOADOPTS"
CC=gfortran %{SOURCE1} liblapacke.so.%{major} liblapacke.a ./liblapack.so.%{major} $LOADOPTS

ar -X64 x liblapacke.so.%{major}

sed -i 's/liblapacke.so.%{major}/liblapacke-indirect.so.0/' shr_64.imp
        
cp shr_64.imp liblapacke.imp


%check
make blas_testing
make cblas_testing

ln -s $(which python2) python
PATH=$PWD:$PATH
make lapack_testing

%install
mkdir -p %{buildroot}%{_includedir}
mkdir -p %{buildroot}%{_libdir}


cp CBLAS/include/*.h %{buildroot}%{_includedir}
cp LAPACKE/include/*.h %{buildroot}%{_includedir}

cp *.so.%{major} %{buildroot}%{_libdir}

cp libblas.imp %{buildroot}%{_libdir}
cp libcblas.imp %{buildroot}%{_libdir}
cp liblapack.imp %{buildroot}%{_libdir}
cp liblapacke.imp %{buildroot}%{_libdir}

# point .so to it's corresponding indirect import file

ln -s libblas.imp %{buildroot}%{_libdir}/libblas.so
ln -s libcblas.imp %{buildroot}%{_libdir}/libcblas.so
ln -s liblapack.imp %{buildroot}%{_libdir}/liblapack.so
ln -s liblapacke.imp %{buildroot}%{_libdir}/liblapacke.so

%post -n libblas%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%postun -n libblas%{major}
%{_bindir}/update-alternatives \
--remove blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

# installing all sub-package alternatives in post and postun scriptlets to get around not having the 
# --add-slave and --remove-slave options in our version of update-alternatives
# Any slave package that is uninstalled will be ignored during install and won't have slave links created

%post -n libcblas%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%postun -n libcblas%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \ 
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%post -n liblapack%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%postun -n liblapack%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%post -n liblapacke%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%postun -n liblapacke%{major}
%{_bindir}/update-alternatives \
--install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libblas.so.%{major} 1 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libcblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/liblapack.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/liblapacke.so.%{major}

%files -n liblapack%{major}
%defattr(-, qsys, *none)

%{_libdir}/liblapack.so.%{major}

%ghost %{_libdir}/liblapack-indirect.so.0

%files devel
%defattr(-, qsys, *none)

%{_libdir}/liblapack.so
%{_libdir}/liblapack.imp


%files -n liblapacke%{major}
%defattr(-, qsys, *none)

%{_libdir}/liblapacke.so.%{major}

%ghost %{_libdir}/liblapacke-indirect.so.0


%files -n lapacke-devel
%defattr(-, qsys, *none)

%{_includedir}/lapacke*

%{_libdir}/liblapacke.so
%{_libdir}/liblapacke.imp


%files -n libblas%{major}
%defattr(-, qsys, *none)

%{_libdir}/libblas.so.%{major}

%ghost %{_libdir}/libblas-indirect.so.0

%files -n blas-devel
%defattr(-, qsys, *none)

%{_libdir}/libblas.so
%{_libdir}/libblas.imp


%files -n libcblas%{major}
%defattr(-, qsys, *none)

%{_libdir}/libcblas.so.%{major}

%ghost %{_libdir}/libcblas-indirect.so.0

%files -n cblas-devel
%defattr(-, qsys, *none)

%{_includedir}/cblas*

%{_libdir}/libcblas.so
%{_libdir}/libcblas.imp


%changelog
* Thu Jan 28 2021 Naveen Ram <naveen.ram@ibm.com> - 3.8.0
- Set up indirects for cblas, lapack, lapacke

* Mon Jan 04 2021 Xu Meng <mengxumx@cn.ibm.com>
- Use new gfortran libaries

* Wed Oct 07 2020 Naveen Ram <naveen.ram@ibm.com> - 3.8.0
- Create blas indirector and implement alternatives

* Thu Nov 28 2019 Kevin Adler <kadler@us.ibm.com> - 3.8.0
- Use SPDX license identifier

* Thu Jan 17 2019 - Xu Meng <mengxumx@cn.ibm.com>
- Build shared libraries instead of static libraries
- Build BLAS libraries from this package, too
- Update to version 3.8.0

* Mon Jun 25 2018 - mengxumx <mengxumx@cn.ibm.com>
- Initial package
